from thread import get_ident, allocate_lock as SimpleLock
from threading import _active
from time import time, sleep

__all__ = ['SimpleLock', 'TimeoutLock', 'RLock']

# the only requirement from locks.Lock is to support blocking 'acquire'


class TimeoutLock(object):
    def __init__(self):
        self.lock = SimpleLock()
        self.release = self.lock.release
        #self.locked = self.lock.locked

    def acquire(self, timeout=None):
        if timeout is None:
            return self.lock.acquire()
        endtime = time() + timeout
        delay = .0005 # initial delay of 1 ms
        while True:
            if self.lock.acquire(False):
                return True
            delay = min(delay * 2, endtime - time(), .05)
            if delay <= 0:
                return False
            sleep(delay)


    __enter__ = acquire

    def __exit__(self, t, v, tb):
        self.release()


class RLock(object):
    def __init__(self):
        self.lock = SimpleLock()
        self.owner = None
        self.depth = 0

    def __repr__(self):
        try:
            owner = _active[self.owner].name
        except KeyError:
            owner = self.owner
        return "<%s owner=%r depth=%d>" % (self.__class__.__name__, owner, self.depth)

    def acquire(self, blocking=True):
        me = get_ident()
        if self.owner == me:
            self.depth += 1
            return 1
        rc = self.lock.acquire(blocking)
        if rc:
            self.owner = me
            self.depth = 1
        return rc

    __enter__ = acquire

    def release(self):
        if self.owner != get_ident():
            raise RuntimeError("cannot release un-acquired lock")
        self.depth -= 1
        if not self.depth:
            self.owner = None
            self.lock.release()

    def __exit__(self, t, v, tb):
        self.release()
