from mext.threads.locks import SimpleLock
from mext.threads.events import MultiEvent, LockedEvent
from mext.classhacks import get_result
from thread import get_ident

__all__ = ['DeferredResult', 'DeferredCall', 'Resource']


_not_set = object()

class DeferredResult(object):
    def __init__(self):
        self.ready = Flag()
        self.wait = self.ready.wait
        self.result = _not_set

    def set(self, value):
        assert self.result is _not_set, "Value is already set"
        self.result = value
        self.ready.set()

    def get(self):
        self.wait()
        return self.result



class DeferredCall(DeferredResult):
    def __init__(self, *args, **kw):
        DeferredResult.__init__(self)
        self.args = args
        self.kw = kw

    def execute(self):
        self.set(get_result(*self.args, **self.kw))

    def get(self):
        return DeferredResult.get(self)()




class Resource(object):
    def __init__(self, count=1, verbose=None):
        if count < 0:
            raise ValueError("Initial resource count must be >= 0")
        self.lock = SimpleLock()
        self.evt_put = LockedEvent(self.lock)
        self.count = count

    def get(self, blocking=True):
        with self.lock:
            while not self.count:
                if not blocking:
                    return False
                self.evt_put.wait()
            self.count -= 1
            return True

    __enter__ = get

    def put(self, count=1):
        with self.lock:
            self.count += count
            self.evt_put.notify(count)

    def __exit__(self, t, v, tb):
        self.put()
